summaryrefslogtreecommitdiff
diff options
context:
space:
mode:
-rw-r--r--django/db/models/deletion.py7
-rw-r--r--tests/modeltests/proxy_model_inheritance/models.py13
-rw-r--r--tests/modeltests/proxy_model_inheritance/tests.py34
3 files changed, 42 insertions, 12 deletions
diff --git a/tests/modeltests/proxy_model_inheritance/models.py b/tests/modeltests/proxy_model_inheritance/models.py
index e69de29bb2..ef9ac6b0d3 100644
--- a/tests/modeltests/proxy_model_inheritance/models.py
+++ b/tests/modeltests/proxy_model_inheritance/models.py
@@ -0,0 +1,13 @@
+
+from django.db import models
+
+
+class ConcreteModel(models.Model):
+ pass
+
+class ConcreteModelSubclass(ConcreteModel):
+ pass
+
+class ConcreteModelSubclassProxy(ConcreteModelSubclass):
+ class Meta:
+ proxy = True
diff --git a/tests/modeltests/proxy_model_inheritance/tests.py b/tests/modeltests/proxy_model_inheritance/tests.py
index ae5ab20b5f..061cdc7e91 100644
--- a/tests/modeltests/proxy_model_inheritance/tests.py
+++ b/tests/modeltests/proxy_model_inheritance/tests.py
@@ -1,11 +1,3 @@
-"""
-XX. Proxy model inheritance
-
-Proxy model inheritance across apps can result in syncdb not creating the table
-for the proxied model (as described in #12286). This test creates two dummy
-apps and calls syncdb, then verifies that the table has been created.
-"""
-
from __future__ import absolute_import
import os
@@ -14,12 +6,20 @@ import sys
from django.conf import settings
from django.core.management import call_command
from django.db.models.loading import cache, load_app
-from django.test import TransactionTestCase
+from django.test import TestCase, TransactionTestCase
from django.test.utils import override_settings
+from .models import (ConcreteModel, ConcreteModelSubclass,
+ ConcreteModelSubclassProxy)
+
@override_settings(INSTALLED_APPS=('app1', 'app2'))
class ProxyModelInheritanceTests(TransactionTestCase):
+ """
+ Proxy model inheritance across apps can result in syncdb not creating the table
+ for the proxied model (as described in #12286). This test creates two dummy
+ apps and calls syncdb, then verifies that the table has been created.
+ """
def setUp(self):
self.old_sys_path = sys.path[:]
@@ -41,3 +41,19 @@ class ProxyModelInheritanceTests(TransactionTestCase):
from .app2.models import NiceModel
self.assertEqual(NiceModel.objects.all().count(), 0)
self.assertEqual(ProxyModel.objects.all().count(), 0)
+
+
+class MultiTableInheritanceProxyTest(TestCase):
+
+ def test_model_subclass_proxy(self):
+ """
+ Deleting an instance of a model proxying a multi-table inherited
+ subclass should cascade delete down the whole inheritance chain (see
+ #18083).
+
+ """
+ instance = ConcreteModelSubclassProxy.objects.create()
+ instance.delete()
+ self.assertEqual(0, ConcreteModelSubclassProxy.objects.count())
+ self.assertEqual(0, ConcreteModelSubclass.objects.count())
+ self.assertEqual(0, ConcreteModel.objects.count())
diff --git a/django/db/models/deletion.py b/django/db/models/deletion.py
index 730847ef21..d8bb8f2e66 100644
--- a/django/db/models/deletion.py
+++ b/django/db/models/deletion.py
@@ -154,9 +154,10 @@ class Collector(object):
model = new_objs[0].__class__
- # Recursively collect parent models, but not their related objects.
- # These will be found by meta.get_all_related_objects()
- for parent_model, ptr in model._meta.parents.iteritems():
+ # Recursively collect concrete model's parent models, but not their
+ # related objects. These will be found by meta.get_all_related_objects()
+ concrete_model = model._meta.concrete_model
+ for ptr in concrete_model._meta.parents.itervalues():
if ptr:
parent_objs = [getattr(obj, ptr.name) for obj in new_objs]
self.collect(parent_objs, source=model,